'----------------------------------------------------------------------------------------
'    Name:  BTR_CAN_BUS_LIB_SUBS.INC
'    Type:  TIGER-BASIC(tm) Source Code
' Release:  V1.00
'
' Purpose:  subroutines for BTR Logline CAN-Bus library
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC-Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'
'----------------------------------------------------------------------------------------
#ifndef	__BTR_CAN_BUS_LIB_SUBS_INC
#define __BTR_CAN_BUS_LIB_SUBS_INC

'***********************************************************************'
'' initializes the BTR Logline CAN-Bus library
sub vLCLInit( )

  byte i

  install_device #CAN, "CAN1_K1.TD2", "00 00 00 00 FF FF FF FF 03 5C 08 1A"%
  ' bgIndexTypeManager(0,0) is not accepted...
  ' so it will be used to store number of registered modules
  bgIndexTypeManager(0,0) = 0
  for i=1 to MAX_NUM_MODULE_ADRESSES - 1 step 1
    bgIndexTypeManager(i,0) = NOT_A_MODULE_ADDRESS ' accepted are 1...99
    bgIndexTypeManager(i,1) = NOT_A_MODULE_TYPE
  next

  for i=0 to LCL_MAX_NUM_MODULES_IN_USE - 1 step 1
    set_len$( sgModuleBuffers$(i), 0 )
  next

  run_task tLCLReadCANBuffer

end
'***********************************************************************'

'***********************************************************************'
'' initializes module of given type and given address
'  bpModType            is it a digital or analog i/o module
'  bpModAddr            module address
sub bLCLInitModule( byte bpModType, bpModAddr; var byte bpvReturn )

  ' wrong input control
  if (bpModType > MAX_MOD_TYPE) or (bpModType = 0) then
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif
  if (bgIndexTypeManager(bpModAddr, 0) + bgIndexTypeManager(bpModAddr, 1)) <> NOT_A_MODULE_ADDRESS then
    bpvReturn = LCL_MODULE_ADDRESS_USED
    return
  endif
  bgIndexTypeManager(bpModAddr, 0) = bgIndexTypeManager(0,0)
  bgIndexTypeManager(bpModAddr, 1) = bpModType
  bgIndexTypeManager(0,0) 	   = bgIndexTypeManager(0,0) + 1
'  bgIndexTypeManager(0,0) stores number of registered modules up to here	
end
'***********************************************************************'

'***********************************************************************'
'' builds the complete standard CAN frame for given values
'  bpModType            is it a digital or analog module
'  bpModAddr            contains the receive module address
'  bpOrderType          0=process data|1=service data|2=control
'  bpRTR                is this a remote frame
'  spData$    		contains the data to send
'  spvStandardFrame$    this frame is created in this function
'  bpvReturn            returns error code (0 for ok)
sub sLCLBuildStandardFrame( byte bpModType, bpModAddr, bpOrderType, &
  bpRTR; string spData$; var string spvStandardFrame$; var byte bpvReturn )

  byte blInfo, blID1, blID2, blDataLen, blModType

  bpvReturn = LCL_OK
  set_len$( spvStandardFrame$, 0 )
  blDataLen = len(spData$)

  ' wrong input control
  if blDataLen > 8 then   '' maximum data length: 8
    bpvReturn = LCL_INVALID_DATA_LENGTH
    return
  endif
  if bpRTR > 1 then
    bpvReturn = LCL_INVALID_RTR_BIT
    return
  endif
  if bpModType > MAX_MOD_TYPE and bpModType = 0 then
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif
  if bpModAddr > 99 then  '' 99 is the maximum module address
    bpvReturn = LCL_INVALID_MODULE_ADDR
    return
  endif
  if bpOrderType > 2 then '' ok: 0=process data|1=service data|2=control
    bpvReturn = LCL_INVALID_ORDER_TYPE
    return
  endif

  ' building the frame
  switchi bpModType
  case FDE4: 	 blModType = LCL_DIGITAL
  case FAE4: 	 blModType = LCL_ANALOGUE
  case FRAS4_21: blModType = LCL_DIGITAL
  case FAA4: 	 blModType = LCL_ANALOGUE
  case FDE10:    blModType = LCL_DIGITAL
  endswitch
  blInfo = (bpRTR shl 6) + blDataLen      '' 0 RTR 0 0 LEN(4*)
  blID1  = (blModType shl 7) + bpModAddr  '' DIG/ANA MODADDR(7*)
  blID2  = (bpOrderType shl 5) + (bpRTR shl 4) + blDataLen
  spvStandardFrame$ = ntos$( spvStandardFrame$, 0, 1, blInfo )
  spvStandardFrame$ = ntos$( spvStandardFrame$, 1, 1, blID1 )
  spvStandardFrame$ = ntos$( spvStandardFrame$, 2, 1, blID2 )
  spvStandardFrame$ = stos$( spvStandardFrame$, 3, spData$, blDataLen )

end
'***********************************************************************'

'***********************************************************************'
'' sends control message to module with given address
'' control message is used to hold module active
'  bpModAddr            contains the receive module address
'  bpvReturn            returns error code (0 for ok)
sub bLCLSendControlMsg( byte bpModAddr; var byte bpvReturn )

  byte blModType, blRTR, blOrderType, blSecondaryAdress
  string slStandardFrame$(LCL_LEN_STANDARD_FRAME), slData$(LCL_MAX_LEN_DATA)

  bpvReturn         = LCL_OK
  blModType         = bgIndexTypeManager(bpModAddr, 1)
  blRTR             = LCL_RTR_NO
  blOrderType 	    = LCL_OT_CONTROL_DATA
  blSecondaryAdress = LCL_RTR_NO       ' secondary address 0 for no RTR
  set_len$( slData$, 0 )

  if blModType = NOT_A_MODULE_TYPE then  ' is module registered
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif

  call sLCLBuildStandardFrame( blModType, bpModAddr, blOrderType, &
  blRTR, slData$, slStandardFrame$, bpvReturn )

  if bpvReturn > LCL_OK then
    return
  endif

  call vSendStandardFrame( slStandardFrame$, blSecondaryAdress )

end
'***********************************************************************'

'***********************************************************************'
'' sends digital information to 4 outputs of module with given address
'' used for FRAS 4/21
'  bpModAddr            contains the receive module address
'  bpDigit1             information for first output (1 or 0)
'  bpDigit2             information for second output (1 or 0)
'  bpDigit3             information for third output (1 or 0)
'  bpDigit4             information for fourth output (1 or 0)
'  bpvReturn            returns error code (0 for ok)
sub bLCLDigOutProcData( byte bpModAddr, &
  bpDigit1, bpDigit2, bpDigit3, bpDigit4; var byte bpvReturn )

  byte blModType, blRTR, blOrderType, blSecondaryAdress, blData
  string slStandardFrame$(LCL_LEN_STANDARD_FRAME), slData$(LCL_MAX_LEN_DATA)

  bpvReturn         = LCL_OK
  blModType         = bgIndexTypeManager(bpModAddr, 1)
  blRTR             = LCL_RTR_NO
  blOrderType 	    = LCL_OT_PROCESS_DATA
  blSecondaryAdress = LCL_RTR_NO       ' secondary address 0 for no RTR
  set_len$( slData$, 0 )

  if blModType = NOT_A_MODULE_TYPE then  ' is module registered
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif

  ' wrong input control
  if ((bpDigit1 > 1) or (bpDigit2 > 1) or (bpDigit3 > 1) or (bpDigit4 > 1)) then
    bpvReturn = LCL_INVALID_DATA_BIT
    return
  endif

  blData = bpDigit1 + (bpDigit2 shl 1) + (bpDigit3 shl 2) + (bpDigit4 shl 3)
  slData$ = ntos$( slData$, 0, 1, blData )

  call sLCLBuildStandardFrame( blModType, bpModAddr, blOrderType, &
  blRTR, slData$, slStandardFrame$, bpvReturn )

  if bpvReturn > LCL_OK then
    return
  endif

  call vSendStandardFrame( slStandardFrame$, blSecondaryAdress )

end
'***********************************************************************'

'***********************************************************************'
'' sends analog information to 4 outputs of module with given address
'' needs 4 unshifted 10-bit-values
'' used for FAA 4
'  bpModAddr            contains the receive module address
'  wpValue1             information for first output
'  wpValue2             information for second output
'  wpValue3             information for third output
'  wpValue4             information for fourth output
'  bpvReturn            returns error code (0 for ok)
sub bLCLAnaOutProcData( byte bpModAddr; &
  word wpValue1, wpValue2, wpValue3, wpValue4; var byte bpvReturn )

  byte blModType, blRTR, blOrderType, blSecondaryAdress
  string slStandardFrame$(LCL_LEN_STANDARD_FRAME), slData$(LCL_MAX_LEN_DATA)

  bpvReturn         = LCL_OK
  blModType         = bgIndexTypeManager(bpModAddr, 1)
  blRTR             = LCL_RTR_NO
  blOrderType 	    = LCL_OT_PROCESS_DATA
  blSecondaryAdress = LCL_RTR_NO       ' secondary address 0 for no RTR
  set_len$( slData$, 0 )

  if blModType = NOT_A_MODULE_TYPE then  ' is module registered
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif

  ' wrong input control
  if ((wpValue1 > 1023) or (wpValue2 > 1023) or (wpValue3 > 1023) or (wpValue4 > 1023)) then
    bpvReturn = LCL_INVALID_ANALOGUE_VALUE
    return
  endif

  slData$ = ntos$( slData$, 0, 2, wpValue1 shl 5 )
  slData$ = ntos$( slData$, 2, 2, wpValue2 shl 5 )
  slData$ = ntos$( slData$, 4, 2, wpValue3 shl 5 )
  slData$ = ntos$( slData$, 6, 2, wpValue4 shl 5 )

  call sLCLBuildStandardFrame( blModType, bpModAddr, blOrderType, &
  blRTR, slData$, slStandardFrame$, bpvReturn )

  if bpvReturn > LCL_OK then
    return
  endif

  call vSendStandardFrame( slStandardFrame$, blSecondaryAdress )

end
'***********************************************************************'

'***********************************************************************'
'' sends a RTR message for service data to module with given address and type
'  bpModAddr            contains the receive module address
'  bpvReturn            returns error code (0 for ok)
sub bLCLSendServRTRMsg( byte bpModAddr; var byte bpvReturn )

  byte blModType, blRTR, blOrderType, blSecondaryAdress
  string slStandardFrame$(LCL_LEN_STANDARD_FRAME), slData$(LCL_MAX_LEN_DATA)

  bpvReturn         = LCL_OK
  blModType         = bgIndexTypeManager(bpModAddr, 1)
  blRTR             = LCL_RTR_YES
  blOrderType 	    = LCL_OT_SERVICE_DATA
  blSecondaryAdress = LCL_RTR_YES       ' secondary address 1 for RTR
  set_len$( slData$, 0 )

  if blModType = NOT_A_MODULE_TYPE then  ' is module registered
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif

  switchi blModType
  case FDE4: 	 slData$ = fill$( "<0>", 2 )  ' fill with 2 dummy bytes
  case FAE4: 	 slData$ = fill$( "<0>", 1 )  ' fill with 1 dummy byte
  case FRAS4_21: slData$ = fill$( "<0>", 2 )  ' fill with 2 dummy bytes
  case FAA4: 	 slData$ = fill$( "<0>", 1 )  ' fill with 1 dummy byte
  case FDE10:    slData$ = fill$( "<0>", 1 )  ' fill with 1 dummy byte
  endswitch

  call sLCLBuildStandardFrame( blModType, bpModAddr, blOrderType, &
  blRTR, slData$, slStandardFrame$, bpvReturn )

  if bpvReturn > LCL_OK then
    return
  endif

  call vSendStandardFrame( slStandardFrame$, blSecondaryAdress )

end
'***********************************************************************'

'***********************************************************************'
'' sends a RTR message for process data to module with given address and type
'  bpModAddr            contains the receive module address
'  bpvReturn            returns error code (0 for ok)
sub bLCLSendProcRTRMsg( byte bpModAddr; var byte bpvReturn )

  byte blModType, blRTR, blOrderType, blSecondaryAdress
  string slStandardFrame$(LCL_LEN_STANDARD_FRAME), slData$(LCL_MAX_LEN_DATA)

  bpvReturn         = LCL_OK
  blModType         = bgIndexTypeManager(bpModAddr, 1)
  blRTR             = LCL_RTR_YES
  blOrderType 	    = LCL_OT_PROCESS_DATA
  blSecondaryAdress = LCL_RTR_YES       ' secondary address 1 for RTR
  set_len$( slData$, 0 )

  if blModType = NOT_A_MODULE_TYPE then  ' is module registered
    bpvReturn = LCL_INVALID_MODULE_TYPE
    return
  endif

  switchi blModType
  case FDE4: 	 slData$ = fill$( "<0>", 1 )  ' fill with 1 dummy byte
  case FAE4: 	 slData$ = fill$( "<0>", 8 )  ' fill with 8 dummy bytes
  case FRAS4_21: slData$ = fill$( "<0>", 1 )  ' fill with 1 dummy byte
  case FAA4: 	 slData$ = fill$( "<0>", 8 )  ' fill with 8 dummy bytes
  case FDE10:    slData$ = fill$( "<0>", 2 )  ' fill with 2 dummy bytes
  endswitch

  call sLCLBuildStandardFrame( blModType, bpModAddr, blOrderType, &
  blRTR, slData$, slStandardFrame$, bpvReturn )

  if bpvReturn > LCL_OK then
    return
  endif

  call vSendStandardFrame( slStandardFrame$, blSecondaryAdress )

end
'***********************************************************************'

'***********************************************************************'
'' sends a given standard frame using given secondary address
'  spStandardFrame$     this string will be send
'  bpSecondaryAddress   using this secondary address
sub vSendStandardFrame( string spStandardFrame$; byte bpSecondaryAdress )
  put #CAN, #bpSecondaryAdress, spStandardFrame$
end
'***********************************************************************'

'***********************************************************************'
'' gets service data of given module address
'  bpModAddr            contains the receive module address
'  bpvModType           returns module type
'  bpvVersion           returns module version number (1 ... 15)
'  bpvSwitches          returns switch states if module is FRAS4/21 type
'  bpvReturn            returns error code (0 for ok)
sub bLCLGetServiceData( byte bpModAddr; var byte bpvModType, bpvVersion, bpvSwitches, bpvReturn )

  byte blOrderType, blTmp, blModTypeIntern
  string slInfo$(LCL_LEN_STANDARD_FRAME)

  bpvReturn = LCL_OK
  blOrderType = LCL_OT_SERVICE_DATA
  blModTypeIntern = bgIndexTypeManager(bpModAddr, 1)

  call sLCLReadDataOfModule( bpModAddr, blOrderType, slInfo$, bpvReturn )
  if (bpvReturn <> LCL_OK) or (len( slInfo$ ) = 0) then     ' error occured
    return                                                  ' or no such data
  endif

  blTmp = nfroms( slInfo$, 3, 1 )
  bpvVersion = blTmp bitand 0fH
  bpvModType = (blTmp bitand 0f0H) shr 4
  if blModTypeIntern = FRAS4_21 then
    if bpvModType = 0bH then     ' 0bH is module's value for FRAS4_21
      bpvSwitches = nfroms( slInfo$, 4, 1 )
    else
      bpvReturn = LCL_WRONG_MODULE_TYPE
      return
    endif
  endif
  if blModTypeIntern = FDE4 then
    if bpvModType = 0aH then     ' 0aH is module's value for FDE4_21
      bpvSwitches = nfroms( slInfo$, 4, 1 )
    else
      bpvReturn = LCL_WRONG_MODULE_TYPE
      return
    endif
  endif
end
'***********************************************************************'

'***********************************************************************'
'' gets process data of given module address
'' do not use for FDE10 module type !!!
'  bpModAddr            contains the receive module address
'  lpvVal1		first process value
'  lpvVal2		second process value
'  lpvVal3		third process value
'  lpvVal4		fourth process value
'  bpvReturn            returns error code (0 for ok)
sub bLCLGetProcessData( byte bpModAddr; var long lpvVal1, lpvVal2, lpvVal3, lpvVal4; var byte bpvReturn )

  byte blOrderType, blModType, blTmp
  string slInfo$(LCL_LEN_STANDARD_FRAME)

  bpvReturn = LCL_OK
  blOrderType = LCL_OT_PROCESS_DATA
  blModType = bgIndexTypeManager(bpModAddr, 1)

  call sLCLReadDataOfModule( bpModAddr, blOrderType, slInfo$, bpvReturn )
  if (bpvReturn <> LCL_OK) or (len( slInfo$ ) = 0) then     ' error occured
    return                                                  ' or no such data
  endif

  switchi blModType
  case FDE4:
    blTmp = nfroms( slInfo$, 3, 1 )
    lpvVal1 = (blTmp bitand 00000001b)
    lpvVal2 = (blTmp bitand 00000010b) shr 1
    lpvVal3 = (blTmp bitand 00000100b) shr 2
    lpvVal4 = (blTmp bitand 00001000b) shr 3
  case FAE4:
    lpvVal1 = nfroms( slInfo$, 3, 2 ) shr 5
    lpvVal2 = nfroms( slInfo$, 5, 2 ) shr 5
    lpvVal3 = nfroms( slInfo$, 7, 2 ) shr 5
    lpvVal4 = nfroms( slInfo$, 9, 2 ) shr 5
  case FRAS4_21:
    blTmp = nfroms( slInfo$, 3, 1 )
    lpvVal1 = (blTmp bitand 00000001b)
    lpvVal2 = (blTmp bitand 00000010b) shr 1
    lpvVal3 = (blTmp bitand 00000100b) shr 2
    lpvVal4 = (blTmp bitand 00001000b) shr 3
  case FAA4:
    lpvVal1 = nfroms( slInfo$, 3, 2 ) shr 5
    lpvVal2 = nfroms( slInfo$, 5, 2 ) shr 5
    lpvVal3 = nfroms( slInfo$, 7, 2 ) shr 5
    lpvVal4 = nfroms( slInfo$, 9, 2 ) shr 5
  case FDE10:
    bpvReturn = LCL_WRONG_MODULE_TYPE
  default:
  endswitch
end
'***********************************************************************'

'***********************************************************************'
'' gets process data of given module address for FDE10 module type
'' do not use for any other module type !!!
'  bpModAddr            contains the receive module address
'  lpvVal1		first process value
'  lpvVal2		second process value
'  lpvVal3		third process value
'  lpvVal4		fourth process value
'  ...                  ...
'  lpvVal10             tenth process value
'  bpvReturn            returns error code (0 for ok)
sub bLCLGetProcessData10( byte bpModAddr; var long lpvVal1, lpvVal2, lpvVal3, lpvVal4, &
  lpvVal5, lpvVal6, lpvVal7, lpvVal8, lpvVal9, lpvVal10; var byte bpvReturn )

  byte blOrderType, blModType, blTmp
  string slInfo$(LCL_LEN_STANDARD_FRAME)

  bpvReturn = LCL_OK
  blOrderType = LCL_OT_PROCESS_DATA
  blModType = bgIndexTypeManager(bpModAddr, 1)

  call sLCLReadDataOfModule( bpModAddr, blOrderType, slInfo$, bpvReturn )
  if (bpvReturn <> LCL_OK) or (len( slInfo$ ) = 0) then     ' error occured
    return                                                  ' or no such data
  endif

  switchi blModType
  case FDE10:
    blTmp = nfroms( slInfo$, 3, 1 )
    lpvVal1  = (blTmp bitand 00000001b)
    lpvVal2  = (blTmp bitand 00000010b) shr 1
    lpvVal3  = (blTmp bitand 00000100b) shr 2
    lpvVal4  = (blTmp bitand 00001000b) shr 3
    lpvVal5  = (blTmp bitand 00010000b) shr 4
    lpvVal6  = (blTmp bitand 00100000b) shr 5
    lpvVal7  = (blTmp bitand 01000000b) shr 6
    lpvVal8  = (blTmp bitand 10000000b) shr 7
    blTmp = nfroms( slInfo$, 4, 1)
    lpvVal9  = (blTmp bitand 00000001b)
    lpvVal10 = (blTmp bitand 00000010b) shr 1
  default:
    bpvReturn = LCL_WRONG_MODULE_TYPE
  endswitch

  end
'***********************************************************************'

'***********************************************************************'
'' converts 10-bit analogue value in long to temperature or voltage value in real
'  lpValueIn            this value will be converted
'  bpValueType          0 for temperature -50/+150, 1 for temperature 0/+400, for2 for voltage
'  rpvValueOut          this value returns the converted value
'  bpvReturn            returns error code (0 for ok)
sub rLCLConvertAnalogueValToReal( long lpValueIn; byte bpValueType; var real rpvValueOut; var byte bpvReturn )

  bpvReturn = LCL_OK

  if lpValueIn > MAX_10BIT_VALUE or lpValueIn < 0 then
    bpvReturn = LCL_VALUE_OUT_OF_RANGE
    return
  endif

  switchi bpValueType
  case LCL_TEMPERATURE_50_150: ' from -50 to +150 degree Celsius
    rpvValueOut = 0.2 * lpValueIn - 52 ' (0=-52C|1023=152,6C)
  case LCL_TEMPERATURE_0_400:  ' from 0 to +400 degree Celsius
    rpvValueOut = 0.4 * lpValueIn       ' (0=0C|1023=409,2C)
  case LCL_VOLTAGE:            ' from 0 to 10 Volt DC
    rpvValueOut = lpValueIn / 100      ' (0=0V|1023=10,23V)
  default:
    bpvReturn = LCL_INVALID_VALUE_TYPE
  endswitch

end
'***********************************************************************'

'***********************************************************************'
'' converts temperature or voltage value in real to 10-bit analogue value in long
'  rpValueIn            this value will be converted
'  bpValueType          0 for temperature -50/+150, 1 for temperature 0/+400, for2 for voltage
'  lpvValueOut          this value returns the converted value
'  bpvReturn            returns error code (0 for ok)
sub lLCLConvertRealToAnalogueVal( real rpValueIn; byte bpValueType; var long lpvValueOut; var byte bpvReturn )

  bpvReturn = LCL_OK

  switchi bpValueType
  case LCL_TEMPERATURE_50_150: ' from -50 to +150 degree Celsius
    if rpValueIn > 150.0 or rpValueIn < -50.0 then
      bpvReturn = LCL_VALUE_OUT_OF_RANGE
      return
    endif
    lpvValueOut = 5 * rpValueIn + 210   ' (-52C=0|152,6C=1023)
  case LCL_TEMPERATURE_0_400:  ' from 0 to +400 degree Celsius
    if rpValueIn > 400.0 or rpValueIn < 0.0 then
      bpvReturn = LCL_VALUE_OUT_OF_RANGE
      return
    endif
    lpvValueOut = 2.5 * rpValueIn       ' (0C=0|409,2C=1023)
  case LCL_VOLTAGE:            ' from 0 to 10 Volt DC
    if rpValueIn > 10.0 or rpValueIn < 0.0 then
      bpvReturn = LCL_VALUE_OUT_OF_RANGE
      return
    endif
    lpvValueOut = rpValueIn * 100      ' (0V=0|10,23V=1023)
  default:
    bpvReturn = LCL_INVALID_VALUE_TYPE
  endswitch

end
'***********************************************************************'

'***********************************************************************'
'' reads information of the module buffer strings and returns it
'  bpModAddr            contains the receive module address
'  bpOrderType          0=process data|1=service data|2=control
'  spvInfo$  	        returns the information from the module buffer
'  bpvReturn            returns error code (0 for ok)
sub sLCLReadDataOfModule( byte bpModAddr, bpOrderType; var string spvInfo$; var byte bpvReturn )

  byte blModType, blBufferIndex, blLen, blPos
  string slTmp$(LCL_LEN_STANDARD_FRAME)

  bpvReturn = LCL_OK
  set_len$( spvInfo$, 0 )
  blBufferIndex = bgIndexTypeManager(bpModAddr, 0)
  blModType = bgIndexTypeManager(bpModAddr, 1)
  blPos = 0

  while (len( spvInfo$ ) = 0) and (bpvReturn = LCL_OK)

    if len( sgModuleBuffers$(blBufferIndex) ) <= blPos then
      return           '' OK exit no such data in buffer
    else
      if len( sgModuleBuffers$(blBufferIndex) ) >= 0.9 * LCL_SIZE_MODULE_BUFFER then
        set_len$( sgModuleBuffers$(blBufferIndex), 0 ) '' clear buffer ...
        return      '' better loose some old values than loose connection
      endif
    endif

    blLen = nfroms( sgModuleBuffers$(blBufferIndex), blPos, 1 ) bitand 0fH

    switchi blLen
    case 0: slTmp$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 3 )  ' control frame - unused
    case 1:  ' FRAS4_21 PROCESS --- FAA4/FAE4 SERVICE --- FDE4 PROCESS
      switchi blModType
      case FDE4:
        if bpOrderType = LCL_OT_PROCESS_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 4 )
        else                                                                  ' no process data requested
          blPos = blPos + 4                                                   ' step to next buffered data
        endif
      case FAE4:
        if bpOrderType = LCL_OT_SERVICE_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 4 )
        else                                                                  ' no service data requested
          blPos = blPos + 4                                                   ' step to next buffered data
        endif
      case FRAS4_21:         				
        if bpOrderType = LCL_OT_PROCESS_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 4 )
        else                                                                  ' no process data requested
          blPos = blPos + 4                                                   ' step to next buffered data
        endif
      case FAA4:
        if bpOrderType = LCL_OT_SERVICE_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 4 )
        else                                                                  ' no service data requested
          blPos = blPos + 4                                                   ' step to next buffered data
        endif
      case FDE10:
        if bpOrderType = LCL_OT_SERVICE_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 4 )
        else                                                                  ' no service data requested
          blPos = blPos + 4                                                   ' step to next buffered data
        endif
      endswitch
    case 2:  ' FRAS4_21 SERVICE --- FDE10 PROCESS --- FDE4 SERVICE
      switchi blModType
      case FDE4:
        if bpOrderType = LCL_OT_SERVICE_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 5 )
        else
          blPos = blPos + 5                                                   ' no service data requested
        endif                                                                 ' step to next buffered data
      case FRAS4_21:          				
        if bpOrderType = LCL_OT_SERVICE_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 5 )
        else                                                                  ' no service data requested
          blPos = blPos + 5                                                   ' step to next buffered data
        endif
      case FDE10:           				
        if bpOrderType = LCL_OT_PROCESS_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 5 )
        else                                                                  ' no process data requested
          blPos = blPos + 5                                                   ' step to next buffered data
        endif
      endswitch
    case 8:  ' ANALOGUE PROCESS DATA
      if blModType = FAE4 or blModType = FAA4 then                            ' only for analogue modules
        if bpOrderType = LCL_OT_PROCESS_DATA then
          spvInfo$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 11 )
        else
          blPos = blPos + 11                                                   ' no process data requested
        endif                                                                 ' step to next buffered data
      else
        bpvReturn = LCL_WRONG_MODULE_TYPE
        slTmp$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, 11 )
      endif	
    default: slTmp$ = cut_and_paste$( sgModuleBuffers$(blBufferIndex), blPos, (blLen + 3) ) ' fatal buffer error
    endswitch

  endwhile

end
'***********************************************************************'

'***********************************************************************'
'' this task permanently polls the CAN Buffer
'' it also writes the CAN Buffer's information
'' into every modules' buffer string
task tLCLReadCANBuffer

  string slMessage$(LCL_LEN_STANDARD_FRAME), slTmp$(LCL_MAXLEN_STANDARD_FRAME_MSG)
  word wlFill
  byte blInfo, blLen, blModAddr, blBufferIndex
  long llId

  while 1 = 1
    get #CAN, #0, #UFCI_IBU_FILL, 0, wlFill
    if wlFill > 2 then
      set_len$( slMessage$, 0 )
      set_len$( slTmp$, 0 )
      get #CAN, #0, 1, blInfo
      slMessage$ = ntos$( slMessage$, 0, 1, blInfo )
      blLen = blInfo bitand 0fH
      if blInfo bitand 80H = 0 then  	'' is it standard frame?
	get #CAN, #0, 2, llId
        slMessage$ = ntos$( slMessage$, 1, 2, llId )
      else                           	'' or extended frame?
	get #CAN, #0, 4, llId
        slMessage$ = ntos$( slMessage$, 1, 4, llId )
      endif
      if blLen > 0 then
        get #CAN, #0, blLen, slTmp$
        slMessage$ = slMessage$ + slTmp$	
        blModAddr = llId bitand 7fH
        blBufferIndex = bgIndexTypeManager(blModAddr, 0)
        if blModAddr <= HIGHEST_MODULE_ADDRESS and blBufferIndex <> NOT_A_MODULE_ADDRESS then    '' only 1 to 99 are allowed
          disable_tsw
          if (len( sgModuleBuffers$(blBufferIndex) ) + LCL_LEN_STANDARD_FRAME) < LCL_SIZE_MODULE_BUFFER then
            sgModuleBuffers$(blBufferIndex) = sgModuleBuffers$(blBufferIndex) + slMessage$
          endif
          enable_tsw
        endif
      endif
    else
      release_task
    endif
  endwhile
end
'***********************************************************************'

#endif

