'----------------------------------------------------------------------------------------
'    Name:  TIGERGRAPHICLIBRARYCONF.INC
'    Type:  TIGER-BASIC(tm) Source Code
' Release:  V1.14	
'
' Purpose:  User configurations for the Tiger Graphic Library
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, German
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC-Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'
'----------------------------------------------------------------------------------------
#ifndef __TIGERGRAPHICLIBRARYCONF_INC
#define __TIGERGRAPHICLIBRARYCONF_INC

'==================================================================================
' General Settings
'==================================================================================
'' initializes all variables for program start
'' ==> should be deactivated for development!!
'user_var_init
'' enforces declarations of variables
'' ==> helps avoiding erors caused by wrong types of variable!!
user_var_strict
'' default string size for declarations
'' can be 0 if string length is given for each string declaration
'' defining each string length by declaration saves stack memory
'user_string_size    64
'' size of temporary strings in some string operations
'' combined operations or logical terms could cause errors,
'' if this value is smaller than the used string
user_tempstr_size   9600
'' memory size for all variables and subs in one task
'' a program require this size of ram for each task
'' the tgl runs with additionally 2 tasks
'' graphic fonts   require a minimum stack size of 300 bytes
'' bitmap elements require a minimum stack size of 400 bytes
'' text elements   require a minimum stack size of 700 bytes
'user_stack_size  2k


'==================================================================================
' Master Defines
' If you don't need these features, you could deactivate them to save flash memory
'==================================================================================
#define TGL_ELEMENTS		' activates elements and windows
#define TGL_TOUCHPANEL	 	' activate all touch panel functions
#define TGL_GRAPHIC_FONTS	' activate all graphic font functions
'' For a more detailed choice of the font styles
'' see below section Setting and Choice of Graphic Fonts.
#define TGL_KEYBOARDS		' activate keyboard functions for touch panel
'' For a more detailed choice of the keyboard styles
'' see below section Choice of Keyboards.
#define TGL_RTC_APPLICATIONS	' activate all graphical RTC applications
'' For a more detailed choice of the real time clock application styles
'' see below section Choice of RTC Applications.


'==================================================================================
' Memory for Elements
'==================================================================================
'' maximum number of specific elements
'' maximum value is 65534=0FFFEh!      	
'' required RAM for default = 91220bytes
#define TGL_MAX_NUM_ELEMENTS	         90 '  4 bytes ram for each element
#define TGL_MAX_NUM_GRAPHICS	         20 ' 14 bytes ram for each element
#define TGL_MAX_NUM_LABELS	         40 ' 23 bytes ram for each element
#define TGL_MAX_NUM_BUTTONS	         10 ' 15 bytes ram for each element
#define TGL_MAX_NUM_TEXT_BUTTONS         10 ' 24 bytes ram for each element
#define TGL_MAX_NUM_SLIDERS		 10 ' 43 bytes ram for each element
#define TGL_MAX_NUM_GAUGES		 10 ' 20 bytes ram for each element
#define TGL_MAX_NUM_LISTBOXES		 10 ' 23 bytes ram for each element


'==================================================================================
' Memory for Windows
'==================================================================================
'' maximum number of windows
'' required RAM = 4*TGL_MAX_NUM_WINDOWS
#define TGL_MAX_NUM_WINDOWS		10	
'' memory for element attributes concerning the window
'' required RAM approx. 11 bytes for each placed element in a window
'' for Tiger 1 this value is limited on 7FFCh (~31k)
#define TGL_WINDOW_ATTRIBUTES_LEN	20k	
'' maximal number of blinking elements in one window
'' required RAM = 2*TGL_MAX_NUM_BLINK
#define TGL_MAX_NUM_BLINK		10	
'' maximal number of buttons in one window
'' required RAM = 14*TGL_MAX_NUM_BUTTONS_IN_WINDOW
#define TGL_MAX_NUM_BUTTONS_IN_WINDOW	10
'' maximal number of sliders in one window
'' required RAM = 37*TGL_MAX_NUM_SLIDERS_IN_WINDOW
#define TGL_MAX_NUM_SLIDERS_IN_WINDOW	10


'==================================================================================
' Memory of Graphic Fonts
' Mind activating the master define TGL_GRAPHIC_FONTS when choosing a graphic font
' For Tiger 1 limited on 7FFCh (~31k):
' TGL_TXT_GRAPHIC_TXT_LEN, TGL_MAX_MEM_TEXTS_LEN
'==================================================================================
'' maximal size for text graphics for building text graphic in one go
'' required RAM = (TGL_GRA_WIDTH/8)*TGL_GRA_HEIGHT
#define TGL_GRA_WIDTH			320 ' must be a multiple of 8!
#define TGL_GRA_HEIGHT			240
'' maximal text length building text graphic in one go
#define TGL_TXT_GRAPHIC_TXT_LEN		512	
'' maximal number of created fonts
'' required RAM = 25 * TGL_MAX_NUM_FONTS
#define TGL_MAX_NUM_FONTS		 10	
'' memory pool for texts in elements which are not saved in the flash memory
#define TGL_MAX_MEM_TEXTS_LEN		 10k	
'' determine maximal number of lines for text graphics
'' required RAM = 8*TGL_MAX_NUM_TXT_LINES
#define TGL_MAX_NUM_TXT_LINES            20


'==================================================================================
' Choice of Graphic Fonts
' Mind activating the master define TGL_GRAPHIC_FONTS when choosing a graphic font!
'==================================================================================
'' define as many fonts you will be using
'' TGL_MAX_NUM_FONTS does not limit the number of these defines
''
'' All bitmaps for activated fonts will be placed in the flash memory.
'' The required flash depends mainly on font size, e.g.:
'' 	VALENCIA_8_NORMAL  ~ 9k
''  	VALENCIA_48_NORMAL ~ 155k
''
'' available fonts are:
''
''      Valencia
''              normal         	8,10,11,12,14,18,21,24,36,48
''              bold            8,10,11,12,14,18,21,24,36,48
''              italic          8,10,11,12,14,18,21
''              bold italic     8,10,11,12,14,18,21
''      Tokio
''              normal         	8,10,11,12,14,18,21
''              bold            8,10,11,12,14,18,21
''              italic          8,10,11,12,14,18,21
''              bold italic     8,10,11,12,14,18,21
''      Atlanta
''              normal         	8,10,11,12,14,18,21
''              bold            8,10,11,12,14,18,21
''              italic          8,10,11,12,14,18,21
''              bold italic     8,10,11,12,14,18,21
''      Istanbul
''              normal          8,10,11,12,14,18,21
'' 	Helsinki
''		normal    	7,8,9,10,11,12,14,18,22,26
''		bold 		      10,   12,14,18,22,26,28,32,52,56,60
''	Amsterdam
''		bold		8,11,16,21
''	Stockholm
''		normal		8,10,11,12,14,18,21
''
'' The defines are composed by the fonts name, size and type:
'' #define FONTNAME_FONTSIZE_FONTTYPE
''
#define VALENCIA_8_NORMAL
#define VALENCIA_10_NORMAL
'#define VALENCIA_11_NORMAL
'#define VALENCIA_12_NORMAL
'#define VALENCIA_14_NORMAL
'#define VALENCIA_18_NORMAL
'#define VALENCIA_21_NORMAL
'#define VALENCIA_24_NORMAL
'#define VALENCIA_36_NORMAL
'#define VALENCIA_48_NORMAL
'#define VALENCIA_8_BOLD
'#define VALENCIA_10_BOLD
'#define VALENCIA_11_BOLD
'#define VALENCIA_12_BOLD
'#define VALENCIA_14_BOLD
'#define VALENCIA_18_BOLD
'#define VALENCIA_21_BOLD
'#define VALENCIA_24_BOLD
'#define VALENCIA_36_BOLD
'#define VALENCIA_48_BOLD
'#define VALENCIA_8_ITALIC
'#define VALENCIA_10_ITALIC
'#define VALENCIA_11_ITALIC
'#define VALENCIA_12_ITALIC
'#define VALENCIA_14_ITALIC
'#define VALENCIA_18_ITALIC
'#define VALENCIA_21_ITALIC
'#define VALENCIA_8_BOLD_ITALIC
'#define VALENCIA_10_BOLD_ITALIC
'#define VALENCIA_11_BOLD_ITALIC
'#define VALENCIA_12_BOLD_ITALIC
'#define VALENCIA_14_BOLD_ITALIC
'#define VALENCIA_18_BOLD_ITALIC
'#define VALENCIA_21_BOLD_ITALIC
'#define TOKIO_8_NORMAL
'#define TOKIO_10_NORMAL
'#define TOKIO_11_NORMAL
'#define TOKIO_12_NORMAL
'#define TOKIO_14_NORMAL
'#define TOKIO_18_NORMAL
'#define TOKIO_21_NORMAL
'#define TOKIO_8_BOLD
'#define TOKIO_10_BOLD
'#define TOKIO_11_BOLD
'#define TOKIO_12_BOLD
'#define TOKIO_14_BOLD
'#define TOKIO_18_BOLD
'#define TOKIO_21_BOLD
'#define TOKIO_8_ITALIC
'#define TOKIO_10_ITALIC
'#define TOKIO_11_ITALIC
'#define TOKIO_12_ITALIC
'#define TOKIO_14_ITALIC
'#define TOKIO_18_ITALIC
'#define TOKIO_21_ITALIC
'#define TOKIO_8_BOLD_ITALIC
'#define TOKIO_10_BOLD_ITALIC
'#define TOKIO_11_BOLD_ITALIC
'#define TOKIO_12_BOLD_ITALIC
'#define TOKIO_14_BOLD_ITALIC
'#define TOKIO_18_BOLD_ITALIC
'#define TOKIO_21_BOLD_ITALIC
'#define ATLANTA_8_NORMAL
'#define ATLANTA_10_NORMAL
'#define ATLANTA_11_NORMAL
'#define ATLANTA_12_NORMAL
'#define ATLANTA_14_NORMAL
'#define ATLANTA_18_NORMAL
'#define ATLANTA_21_NORMAL
'#define ATLANTA_8_BOLD
'#define ATLANTA_10_BOLD
'#define ATLANTA_11_BOLD
'#define ATLANTA_12_BOLD
'#define ATLANTA_14_BOLD
'#define ATLANTA_18_BOLD
'#define ATLANTA_21_BOLD
'#define ATLANTA_8_ITALIC
'#define ATLANTA_10_ITALIC
'#define ATLANTA_11_ITALIC
'#define ATLANTA_12_ITALIC
'#define ATLANTA_14_ITALIC
'#define ATLANTA_18_ITALIC
'#define ATLANTA_21_ITALIC
'#define ATLANTA_8_BOLD_ITALIC
'#define ATLANTA_10_BOLD_ITALIC
'#define ATLANTA_11_BOLD_ITALIC
'#define ATLANTA_12_BOLD_ITALIC
'#define ATLANTA_14_BOLD_ITALIC
'#define ATLANTA_18_BOLD_ITALIC
'#define ATLANTA_21_BOLD_ITALIC
'#define ISTANBUL_8_NORMAL
'#define ISTANBUL_10_NORMAL
'#define ISTANBUL_11_NORMAL
'#define ISTANBUL_12_NORMAL
'#define ISTANBUL_14_NORMAL
'#define ISTANBUL_18_NORMAL
'#define ISTANBUL_21_NORMAL
'#define HELSINKI_7_NORMAL
'#define HELSINKI_8_NORMAL
'#define HELSINKI_9_NORMAL
'#define HELSINKI_10_NORMAL
'#define HELSINKI_11_NORMAL
'#define HELSINKI_12_NORMAL
'#define HELSINKI_14_NORMAL
'#define HELSINKI_18_NORMAL
'#define HELSINKI_22_NORMAL
'#define HELSINKI_26_NORMAL
'#define HELSINKI_10_BOLD
'#define HELSINKI_12_BOLD
'#define HELSINKI_14_BOLD
'#define HELSINKI_18_BOLD
'#define HELSINKI_22_BOLD
'#define HELSINKI_26_BOLD
'#define HELSINKI_28_BOLD
'#define HELSINKI_32_BOLD
'#define HELSINKI_52_BOLD
'#define HELSINKI_56_BOLD
'#define HELSINKI_60_BOLD
'#define AMSTERDAM_8_BOLD
'#define AMSTERDAM_11_BOLD
'#define AMSTERDAM_16_BOLD
'#define AMSTERDAM_21_BOLD
'#define STOCKHOLM_8_NORMAL
'#define STOCKHOLM_10_NORMAL
'#define STOCKHOLM_11_NORMAL
'#define STOCKHOLM_12_NORMAL
'#define STOCKHOLM_14_NORMAL
'#define STOCKHOLM_18_NORMAL
'#define STOCKHOLM_21_NORMAL

'' include font
'' must be included AFTER the font defines because they are used inside the font include files
#include TGL_GRAFO_VALENCIA.INC 	' load up to 34 Bitmap FONTs "VALENCIA"     Family of ID-Code = 2
'#include TGL_GRAFO_ATLANTA.INC		' load up to 28 Bitmap FONTs "ATLANTA"      Family of ID-Code = 2
'#include TGL_GRAFO_TOKIO.INC 		' load up to 28 Bitmap FONTs "TOKIO"        Family of ID-Code = 2
'#include TGL_GRAFO_ISTANBUL.INC	' load up to 7  Bitmap FONTs "ISTANBUL"     Family of ID-Code = 2
'#include TGL_GRAFO_HELSINKI.INC	' load up to 21 Bitmap FONTs "HELSINKI"     Family of ID-Code = 2
'#include TGL_GRAFO_AMSTERDAM.INC	' load up to 4  Bitmap FONTs "AMSTERDAM"    Family of ID-Code = 2
'#include TGL_GRAFO_STOCKHOLM.INC       ' load up to 7  Bitmap FONTs "STOCKHOLM"    Family of ID-Code = 2

'==================================================================================
' Choice of Keyboards
'==================================================================================
' For saving ram and flash activate only those keyboard styles you really need
' for your project.
' Depending of the style a keyboard need to load bitmaps in the flash.
' This flash is needed whether you call the keyboard in your project or not.
' Mind activating the master define TGL_KEYBOARDS when calling a keyboard!
#ifdef TGL_KEYBOARDS
  #define TGL_KEYB_DIG_1       	' digit block
  #define TGL_KEYB_1_ENG       	' english style
  '#define TGL_KEYB_1_GER       ' german style
  '#define TGL_KEYB_1_TRK       ' turkey style
  '#define TGL_KEYB_1_HUN       ' hungarian style
#endif


'==================================================================================
' Choice of RTC Applications
'==================================================================================
#ifdef TGL_RTC_APPLICATIONS
  #define TGL_DEF_RTC_STYLE_1	' show and set a digital clock
#endif


'==================================================================================
' HARDWARE
'==================================================================================
' BEEPER
#define PORT_BEEPER		4
#define PIN_BEEPER		2
#define BEEPER_ON		0FFh
#define BEEPER_OFF		0
' LCD
'  - size
#define LCD_WIDTH	  	320	' pixel width
#define LCD_HEIGHT	  	240	' pixel height
'  - ports and pins
#define PORT_LCD		8
#define PIN_LCD_RESET		5
#define LCD_SHUT_DOWN		0
#define LCD_START_UP		0FFh
#define PIN_LCD_BACKLIGHT	2
#define LCD_BACKLIGHT_ON	0
#define LCD_BACKLIGHT_OFF	0FFh
#define BUSY_TIME_LCD		100	' time for LCD for starting up
					' "LCD_BACKGROUND_STANDBY_MODE" is definded before TGL is included
#ifndef LCD_INVERSION_MODE
  #define LCD_INVERSION_MODE	0 	' 0 = normal    for "white" LCD
#endif					' 1 = inversion for "blue"  LCD
' TOUCH PANEL
'#define TP_TYPE		TP_TYP_0' analog device driver is used
#define TP_TYPE  		TP_TYP_1' analog device driver is not used
#define TP_CALIB_EEPROM_ADDR	0	' from this eeprom address on are 26 bytes reserved for touch panel calibration
' EEPROM
#define PORT_EEPROM		7
#define CLK_EEPROM		0
#define DATA_EEPROM		1
#define SPEED_REDUCTION_EEPROM	0


'==================================================================================
' Versions
'===========================================================================
' If you program with the actual version of the Tiger Graphic Library
' all these defines must be activated
' For compiling code which has been written for older versions of the
' Tiger Graphic Library deactivate all defines for newer versions than the
' used version
#define TGL_V1_01
#define TGL_V1_13
#define TGL_V1_14


#endif ' __TIGERGRAPHICLIBRARYCONF_INC


